#!/usr/bin/env python3

# ROS libraries
import rospy
from gazebo_msgs.msg import ModelStates
import math
from people_detection_msgs.msg import PeoplePose

import configparser


# Pre-defined global variables

CONFIG_PATH = "dataset/config/scenario_loop_mini_mini.ini"
_first_msg_received = False

MAX_NB_ROBOTS = 4
_robots_poses = []
_model_names = []
COMM_RANGE = 2 # max distance to allow robot communication

_robots_detection_msgs = []
pubs_detection = []


##############################################################
##############################################################
# Practical session P1.4
# The position and orientation of the robot is given in the
# variable _trhuth_pose
# Your task is to design the path planning algorithm that
# drives the robot to the waypoints signaled in the map, and
# in the shortest possible time.
# To do so, you must drive the robot by checking its position and orientation, and
# send velocity commands accordingly.
# you must set the output _cmd_vel with the proper velocity values.


# You can create 2D pose structures such as
# _waypoint_1 = Pose2D()
# _and assign value to its x and y coordinates, and theta orientation:
# _waypoint_1.x = 1.0 # (m)
# _waypoint_1.y = 1.0 # (m)
# _waypoint_1.theta = 1.0 # (rad)[-pi,pi],
# If you require these or other class of global variables, add them here
# and declare them in the function process_data()



# Insert your code inside the function process_data
def process_data():
    # You can acces the value of the following variables:
    # _truth_pose.x , for the x coordinate of the position of the robot
    # _truth_pose.y , for the y coordinate of the position of the robot
    # _truth_pose.theta, for the orientation of the robot
    # The position and orientation of the robot is computed with
    # respect to the center of the field, in the position (0,0).
    for i in range(MAX_NB_ROBOTS):
        for j in range(i+1,MAX_NB_ROBOTS):
            if in_range(_robots_poses[i], _robots_poses[j]):
                dispatch_detection_msgs(i,j)
                print("exchange between "+str(i)+" and "+str(j))






def in_range(pos1, pos2):
    distance = math.sqrt(((pos1.x-pos2.x)**2)+((pos1.y-pos2.y)**2))
    return distance <= COMM_RANGE

def dispatch_detection_msgs(i,j):
    global _robots_detection_msgs, pubs_detection
    pubs_detection[i].publish(_robots_detection_msgs[j])
    pubs_detection[j].publish(_robots_detection_msgs[i])

# Define callbacks and functions to process incoming messages
def gazebo_callback(msg):
    global _robots_poses, _model_names, _first_msg_received

    if len(_model_names) < MAX_NB_ROBOTS:
        for name in msg.name:
            if "robot" in name:
                _model_names.append(name)
        _model_names = sorted(_model_names)
    _robots_poses.clear()
    for name in _model_names:
        _robots_poses.append(msg.pose[msg.name.index(name)].position)
    if not _first_msg_received:
            _first_msg_received = True

def detection_callback(msg, id):
    _robots_detection_msgs[id] = msg





# Main

if __name__ == '__main__':
    # Add here the name of the ROS node. In ROS, node names must be unique.
    rospy.init_node('features_dispatch_node')

    config = configparser.ConfigParser()
    config.read(CONFIG_PATH)
    MAX_NB_ROBOTS = int(config['Robots']['max_nb_robots'])
    COMM_RANGE = float(config['Robots']['comm_range'])

    for i in range(MAX_NB_ROBOTS):
        _robots_detection_msgs.append(PeoplePose())

    # Subscribe to the topics and associate the correspondign callback functions
    sub_pose = rospy.Subscriber('/gazebo/model_states/', ModelStates, gazebo_callback)
    subs_detection = []
    detection_callback_lambda = lambda x: detection_callback(x,0)
    subs_detection.append(rospy.Subscriber('/r'+str(0)+'/set_people_detected', PeoplePose, detection_callback_lambda))
    detection_callback_lambda = lambda x: detection_callback(x,1)
    subs_detection.append(rospy.Subscriber('/r'+str(1)+'/set_people_detected', PeoplePose, detection_callback_lambda))
    detection_callback_lambda = lambda x: detection_callback(x,2)
    subs_detection.append(rospy.Subscriber('/r'+str(2)+'/set_people_detected', PeoplePose, detection_callback_lambda))
    detection_callback_lambda = lambda x: detection_callback(x,3)
    subs_detection.append(rospy.Subscriber('/r'+str(3)+'/set_people_detected', PeoplePose, detection_callback_lambda))
    
    # Publish messages
    for i in range(MAX_NB_ROBOTS):
        pubs_detection.append(rospy.Publisher('/r'+str(i)+'/get_people_detected', PeoplePose, queue_size = 10))

    rate=rospy.Rate(30)


    while not rospy.is_shutdown():
        if _first_msg_received:
            process_data()
        rate.sleep()
