#!/usr/bin/env python3

# ROS libraries
import rospy
from geometry_msgs.msg import Pose2D
from gazebo_msgs.msg import ModelStates
from sensor_msgs.msg import Image
from cv_bridge import CvBridge
import cv2
import os
import configparser
import re
import random

# Pre-defined global variables

_first_msg_received = False
_pubs_image = []

IMAGE_DIR_PATH = "dataset/extended/"
CONFIG_PATH = "dataset/config/scenario_loop_mini_mini.ini"

MAX_NB_ROBOTS = 0
_robots_poses = []
_model_names = []
_current_robots_locations = []

ROOMS = {}
_current_context_in_room = {}

# Automatons
MAX_NB_AUTOMATONS = 4
TRANSITIONS_LAYOUT = {}
TRANSITIONS_PROB = {}
TRANSITIONS_DELAYS = []
_list_states_automatons = []

# Results
RESULTS_DIR_PATH = "dataset/results/"
ROBOT_FILENAME = "robot_"
AUTOMATON_FILENAME = "automaton_"

##############################################################
##############################################################
# Practical session P1.4
# The position and orientation of the robot is given in the
# variable _trhuth_pose
# Your task is to design the path planning algorithm that
# drives the robot to the waypoints signaled in the map, and
# in the shortest possible time.
# To do so, you must drive the robot by checking its position and orientation, and
# send velocity commands accordingly.
# you must set the output _cmd_vel with the proper velocity values.


# You can create 2D pose structures such as
# _waypoint_1 = Pose2D()
# _and assign value to its x and y coordinates, and theta orientation:
# _waypoint_1.x = 1.0 # (m)
# _waypoint_1.y = 1.0 # (m)
# _waypoint_1.theta = 1.0 # (rad)[-pi,pi],
# If you require these or other class of global variables, add them here
# and declare them in the function process_data()



# Insert your code inside the function process_data
def process_data():
    # You can acces the value of the following variables:
    # _truth_pose.x , for the x coordinate of the position of the robot
    # _truth_pose.y , for the y coordinate of the position of the robot
    # _truth_pose.theta, for the orientation of the robot
    # The position and orientation of the robot is computed with
    # respect to the center of the field, in the position (0,0).
    global _robots_poses, _pubs_image
    for i in range(MAX_NB_ROBOTS):
        pose = (_robots_poses[i].position.x, _robots_poses[i].position.y)
        location = get_location_from_pose(pose)
        if location[0] != "Unknown":
            if location[0] != _current_robots_locations[i]:
                _current_robots_locations[i] = location[0]
                filename = ROBOT_FILENAME+str(i)
                write_results(filename, location[0])
            img = get_image_by_position(location)
            if not img is None:
                bridge = CvBridge()
                img_msg = bridge.cv2_to_imgmsg(img, encoding="passthrough")
                _pubs_image[i].publish(img_msg)
            else:
                print("Error: Image not found.")
        else:
             print("Error: Not in a room.")


def get_location_from_pose(pose):
    location = ["Unknown", 0]      # location[1] values go from 1 to 4 (one for each quadrant/quarter of the room)
    for room, vertices in ROOMS.items():
        if is_inside_rectangle(vertices, pose):
            location[0] = room
            if "corridor" in room:
                location[1] = get_quarter_from_pose(vertices, pose)
            else:
                location[1] = get_quadrant_from_pose(vertices, pose)
    return location

def get_image_by_position(location):
    global _current_context_in_room
    img = None
    room = location[0]
    if "corridor" in location[0]:
         room = "corridor"
    path = IMAGE_DIR_PATH+room+"/p"+str(location[1])+"/"+_current_context_in_room[location[0]]+"/"
    img_files = [f for f in os.listdir(path) if f.endswith(".png")]
    if not img_files:
        print("Error: No images found in the directory.")
    else:
        random_index = random.randint(0, len(img_files) - 1)
        img_name = img_files[random_index]
        path += img_name
    if os.path.isfile(path):
        img = cv2.imread(path)
    return img


def dot_prod_with_shared_start(start, end1, end2):
    """
    Compute the dot product of the vectors pointing from start to end1 and end2

    Parameters
        start: starting point of both vectors
        end1: end point of first vector
        end2: end point of second vector

    Returns
        dot product of (end1 - start) and (end2 - start)
    """
    return (end1[0] - start[0]) * (end2[0] - start[0]) + (end1[1] - start[1]) * (end2[1] - start[1])

def is_inside_rectangle(vertices, point):
    """
    Given the vertices of a rectangle, determine whether a point is inside it.

    Parameters
        vertices: a list of tuples representing rectangle vertices in clockwise or
                  counter-clockwise order
        point: a tuple representing the point to check

    Returns
        True if the point is inside the rectangle. False otherwise.
    """
    return all(dot_prod_with_shared_start(vertices[i - 1], v, point) >= 0 for i, v in enumerate(vertices))

def get_quadrant_from_pose(vertices, pose):
    quadrant_id = 0
    quadrants = get_quadrants(vertices)
    if is_inside_rectangle(quadrants[0], pose): 
            quadrant_id = 1
    elif is_inside_rectangle(quadrants[1], pose): 
            quadrant_id = 2
    elif is_inside_rectangle(quadrants[2], pose): 
            quadrant_id = 3
    elif is_inside_rectangle(quadrants[3], pose): 
            quadrant_id = 4
    return quadrant_id

def get_quadrants(vertices):
    center = ((vertices[0][0]+vertices[2][0])/2, (vertices[0][1]+vertices[2][1])/2)
    mid1 = ((vertices[0][0]+vertices[3][0])/2, (vertices[0][1]+vertices[3][1])/2)
    mid2 = ((vertices[0][0]+vertices[1][0])/2, (vertices[0][1]+vertices[1][1])/2)
    mid3 = ((vertices[1][0]+vertices[2][0])/2, (vertices[1][1]+vertices[2][1])/2)
    mid4 = ((vertices[2][0]+vertices[3][0])/2, (vertices[2][1]+vertices[3][1])/2)
    quadrant1 = [vertices[0], mid2, center, mid1]
    quadrant2 = [mid2, vertices[1], mid3, center]
    quadrant3 = [center, mid3, vertices[2], mid4]
    quadrant4 = [mid1, center, mid4, vertices[3]]
    return [quadrant1, quadrant2, quadrant3, quadrant4]

def get_quarter_from_pose(vertices, pose):
    quartert_id = 0
    quarters = get_quarters(vertices)
    if is_inside_rectangle(quarters[0], pose): 
            quartert_id = 1
    elif is_inside_rectangle(quarters[1], pose): 
            quartert_id = 2
    elif is_inside_rectangle(quarters[2], pose): 
            quartert_id = 3
    elif is_inside_rectangle(quarters[3], pose): 
            quartert_id = 4
    return quartert_id
    
def get_quarters(vertices):
    mid1 = ((vertices[0][0]+vertices[3][0])/2, (vertices[0][1]+vertices[3][1])/2)
    mid2 = ((vertices[1][0]+vertices[2][0])/2, (vertices[1][1]+vertices[2][1])/2)
    mid11 = ((vertices[0][0]+mid1[0])/2, (vertices[0][1]+mid1[1])/2)
    mid12 = ((vertices[3][0]+mid1[0])/2, (vertices[3][1]+mid1[1])/2)
    mid21 = ((vertices[1][0]+mid2[0])/2, (vertices[1][1]+mid2[1])/2)
    mid22 = ((vertices[2][0]+mid2[0])/2, (vertices[2][1]+mid2[1])/2)
    quarter1 = [vertices[0], vertices[1], mid21, mid11]
    quarter2 = [mid11, mid21, mid2, mid1]
    quarter3 = [mid1, mid2, mid22, mid21]
    quarter4 = [mid12, mid22, vertices[2], vertices[3]]
    return [quarter1, quarter2, quarter3, quarter4]


##############################################################
##############################################################

def get_new_state(state):
    n = random.random()
    if n <= TRANSITIONS_PROB[state]:
        i = random.randint(0,len(TRANSITIONS_LAYOUT[state])-1)
        state = TRANSITIONS_LAYOUT[state][i]
    return state

def update_context():
    global _list_states_automatons, _current_context_in_room
    for room in _current_context_in_room:
        _current_context_in_room[room] = "4"
    for i in range(len(_list_states_automatons)):
        room = _list_states_automatons[i]
        if _current_context_in_room[room] == "4":
            _current_context_in_room[room] = str(i)
        else:
            _current_context_in_room[room] += str(i)


##############################################################
##############################################################

# Define callbacks and functions to process incoming messages
def gazebo_callback(msg):
    global _robots_poses, _model_names, _first_msg_received

    if len(_model_names) < MAX_NB_ROBOTS:
        for name in msg.name:
            if "robot" in name:
                _model_names.append(name)
        _model_names = sorted(_model_names)
    _robots_poses.clear()
    for name in _model_names:
        _robots_poses.append(msg.pose[msg.name.index(name)])
    if not _first_msg_received:
            _first_msg_received = True

def transition_callback(data, id):
    global _list_states_automatons
    new_state = get_new_state(_list_states_automatons[id])
    filename = AUTOMATON_FILENAME+str(id)
    write_results(filename, new_state)
    if new_state != _list_states_automatons[id]:
        _list_states_automatons[id] = new_state
        update_context()

def setup_scenario():
    global _current_context_in_room, MAX_NB_ROBOTS, MAX_NB_AUTOMATONS, TRANSITIONS_DELAYS, RESULTS_DIR_PATH, ROBOT_FILENAME, AUTOMATON_FILENAME
    config = configparser.ConfigParser()
    config.read(CONFIG_PATH)
    MAX_NB_ROBOTS = int(config['Robots']['max_nb_robots'])
    MAX_NB_AUTOMATONS = int(config['Automatons']['max_nb_automatons'])
    for option in config.options('Rooms'):
        ROOMS[option] = [tuple(float(el2) for el2 in el.strip('()').split(',')) for el in re.findall('\([^)]*\)',config.get('Rooms', option))]
    for option in config.options('Init.Context'):
        _current_context_in_room[option] = config.get('Init.Context', option)
    for option in config.options('Transitions.Layout'):
        TRANSITIONS_LAYOUT[option] = config.get('Transitions.Layout', option).split(',')
    for option in config.options('Transitions.Prob'):
        TRANSITIONS_PROB[option] = float(config.get('Transitions.Prob', option))
    TRANSITIONS_DELAYS = [float(el) for el in config['Transitions.Delays']['delays'].split(',')]
    RESULTS_DIR_PATH = config['Results']['path']
    ROBOT_FILENAME = config['Results']['robot_filename']
    AUTOMATON_FILENAME = config['Results']['automaton_filename']

def write_results(filename, location):
    timestamp = rospy.Time.now()
    file = open(RESULTS_DIR_PATH+filename, "a")
    file.write(str(timestamp.secs) + '.' + str(timestamp.nsecs) + " : " + location + "\n")
    file.close()


# Main

if __name__ == '__main__':

    setup_scenario()
    # Add here the name of the ROS node. In ROS, node names must be unique.
    rospy.init_node('image_dealer_node')

    # Subscribe to the topics and associate the correspondign callback functions
    sub_pose = rospy.Subscriber('/gazebo/model_states/', ModelStates, gazebo_callback)

    # Publish messages
    for i in range(MAX_NB_ROBOTS):
        pub_image = rospy.Publisher('/r'+str(i)+'/image', Image, queue_size = 1)
        _pubs_image.append(pub_image)
        _current_robots_locations.append("Unknown")

    rate=rospy.Rate(30)
    for i in range(MAX_NB_AUTOMATONS):
        automaton_state = "Unknown"
        for room, context in _current_context_in_room.items():
            if str(i) in context:
                automaton_state = room
        _list_states_automatons.append(automaton_state)
        filename = AUTOMATON_FILENAME+str(i)
        write_results(filename, automaton_state)
    
    transition_callback_lambda = lambda x: transition_callback(x,0)
    rospy.Timer(rospy.Duration(TRANSITIONS_DELAYS[0]), transition_callback_lambda)
    transition_callback_lambda = lambda x: transition_callback(x,1)
    rospy.Timer(rospy.Duration(TRANSITIONS_DELAYS[1]), transition_callback_lambda)
    transition_callback_lambda = lambda x: transition_callback(x,2)
    rospy.Timer(rospy.Duration(TRANSITIONS_DELAYS[2]), transition_callback_lambda)
    transition_callback_lambda = lambda x: transition_callback(x,3)
    rospy.Timer(rospy.Duration(TRANSITIONS_DELAYS[3]), transition_callback_lambda)

    while not rospy.is_shutdown():
        if _first_msg_received:
            process_data()
        rate.sleep()
